import React from "react";
import { HeroSection } from "@components/HeroSection";
import { HorizontalNewsCard } from "@components/HorizontalNewsCard";
import Breadcrumbs from "@components/Breadcrumbs";
import { TableOfContents, Headline } from "@components/TableOfContents";
import { Pill } from "@components/Pill";
import { Prose } from "@components/Prose";
import LineSVG from "@assets/images/blog-end-line.svg";
import { INews } from "@interfaces/INews";
import { Link } from "@components/Link";
import { Edit } from "@carbon/icons-react";

interface TableOfContentsItem {
  target: string;
  headline: string;
}

interface Props {
  titleImage?: string;
  children: React.ReactNode;
  subtitle?: string;
  teaser?: string;
  title?: string;
  tableOfContents?: Array<TableOfContentsItem>;
  relatedContent?: Array<INews>;
  dateAndAuthor?: string;
  tags?: Array<{ target?: string; title: string }>;
  notProse?: boolean;
  preContentChildren?: React.ReactNode;
  className?: string;
  hideFooterSeparator?: boolean;
  breadcrumbs?: Array<{ target: string; title: string }>;
  imageTag?: string;
  editButtonTarget?: string;
}

const Article: React.FC<Props> = ({
  titleImage,
  children,
  subtitle,
  title,
  teaser,
  relatedContent,
  tableOfContents,
  dateAndAuthor,
  tags,
  notProse,
  preContentChildren,
  className,
  hideFooterSeparator,
  breadcrumbs,
  imageTag,
  editButtonTarget,
}) => {
  const TEXT_STYLE_MAPPING: Record<string, string> = {
    main: `font-plex text-grey-900 ${notProse ? "" : "prose"}`,
    headings: "prose-headings:font-karla",
    subheading: "prose-h2:text-3xl",
    bullets: "prose-li:marker:text-grey-900",
    blockquote:
      "prose-blockquote:border-l-2 sm:prose-blockquote:border-l-4 prose-blockquote:border-grey-900 prose-blockquote:text-sm sm:prose-blockquote:text-base prose-blockquote:not-italic prose-blockquote:leading-5 sm:prose-blockquote:leading-6 prose-blockquote:font-semibold prose-blockquote:tracking-wider prose-blockquote:uppercase",
    code: "prose-code:before:content-none prose-code:after:content-none",
    image: "prose-img:w-full prose-img:h-auto prose-img:rounded-sm",
    video: "prose-video:w-full prose-video:h-auto",
  };

  return (
    <div className={`w-full mb-10 xl:mb-0 ${className || ""}`}>
      {titleImage && <HeroSection src={titleImage} imageTag={imageTag} />}
      <section>
        <div className="grid grid-cols-article gap-x-4">
          <div className="relative">
            {tableOfContents?.length && (
              <div className="hidden xl:block p-10 absolute right-8">
                <TableOfContents>
                  {tableOfContents.map((tocItem) => (
                    <Headline key={tocItem.headline} target={tocItem.target}>
                      {tocItem.headline}
                    </Headline>
                  ))}
                </TableOfContents>
              </div>
            )}
          </div>

          <div className="w-full">
            <div className="flex justify-between mt-5">
              <Breadcrumbs crumbs={breadcrumbs} />
              {editButtonTarget && <EditButton target={editButtonTarget} />}
            </div>
            <div className="mt-10 text-center">
              {subtitle && (
                <span id="article-subtitle" className="block font-karla text-blue-500 text-base leading-6 font-semibold tracking-wide uppercase mb-4">
                  {subtitle}
                </span>
              )}
              {title && (
                <h2 id="article-title" className="font-karla text-grey-900 text-3xl sm:text-4xl xl:text-6xl leading-none font-bold tracking-tight">
                  {title}
                </h2>
              )}
            </div>
            <div className="mt-10 mb-20 xl:mb-44">
              {preContentChildren}
              <Prose>
                {teaser && <p className="lead">{teaser}</p>}
                {children}
              </Prose>
              <div className="overflow-hidden">
                {!hideFooterSeparator && (
                  <img
                    className="w-full mt-10 mb-6 origin-left scale-[2.5] sm:scale-100"
                    src={LineSVG}
                  />
                )}
                {dateAndAuthor && (
                  <span className="text-grey-900 text-sm leading-4 font-medium">
                    {dateAndAuthor}
                  </span>
                )}
                {tags?.length && (
                  <div className="flex gap-2 mt-2">
                    {tags.map((tag) => (
                      <Pill
                        rounded
                        key={tag.title}
                        type="dark"
                        size="sm"
                        title={tag.title}
                        href={tag.target}
                      />
                    ))}
                  </div>
                )}
              </div>
            </div>
            {relatedContent?.length && (
              <div>
                <h2 className="font-karla md:font-plex text-black text-2xl leading-7 font-bold md:text-3xl md:leading-9">
                  Related Content
                </h2>
                <div>
                  {relatedContent.map((news, index) => {
                    return (
                      index < 3 && (
                        <HorizontalNewsCard
                          key={news.id}
                          type={news.type}
                          image={news.image}
                          title={news.title}
                          description={news.description}
                          time={news.time}
                          location={news.location}
                          category={news.category}
                          href={news.href}
                          className="mt-7"
                        />
                      )
                    );
                  })}
                </div>
              </div>
            )}
          </div>
          <div className="hidden xl:block"></div>
        </div>
      </section>
    </div>
  );
};

export default Article;

interface EditButtonProps {
  target: string;
  className?: string;
}

const EditButton: React.FC<EditButtonProps> = ({ target, className }) => {
  return (
    <div
      className={`hidden sm:block font-inter font-normal text-sm leading-5 text-grey-500 ${
        className || ""
      }`}
    >
      <Link
        to={target}
        className={`flex-shrink-0 !flex gap-[5px] items-center ${
          target ? "hover:multi-line-underline multi-line-grey-500 !w-auto" : ""
        }`}
      >
        <Edit />
        Edit Page
      </Link>
    </div>
  );
};
