import React, { CSSProperties, useState } from "react";
import { ChevronRight } from "@carbon/icons-react";
import { ChevronLeft } from "@carbon/icons-react";
import { VerticalNewsCard } from "@components/VerticalNewsCard";
import { INewsCard, INews, Color } from "@interfaces/INews";
import { useKeenSlider } from "keen-slider/react";

type Props = {
  className?: string;
  title?: string;
  children?: React.ReactNode;
  items?: Array<INews | INewsCard>;
  customBackgroundColor?: Color;
  textColor?: "dark" | "light";
  fullBleed?: boolean;
};

const Carousel: React.FC<Props> = ({
  className = "",
  title,
  items,
  customBackgroundColor,
  textColor,
  fullBleed,
}) => {
  const [currentIndex, setCurrentIndex] = useState(0);
  const [loaded, setLoaded] = useState(false);
  const [sliderRef, instanceRef] = useKeenSlider<HTMLDivElement>({
    selector: ".slide",
    mode: "free-snap",
    slides: {
      origin: "center",
      perView: "auto",
      spacing: 15,
    },
    initial: 0,
    slideChanged(slider) {
      setCurrentIndex(slider.track.details.rel);
    },
    created() {
      setLoaded(true);
    },
  });

  const fullBleedClasses: CSSProperties = fullBleed
    ? {
        width: "100vw",
        position: "relative",
        marginLeft: "-50vw",
        left: "50%",
      }
    : {};

  return (
    <div className={className}>
      {title && (
        <div className="container">
          <h3 className="font-inter sm:font-karla text-3xl md:text-5xl leading-9 md:leading-none font-normal text-grey-900 mb-6 md:mb-8">
            {title}
          </h3>
        </div>
      )}
      <div
        style={fullBleedClasses}
        className={`relative ${fullBleed ? "" : "overflow-hidden"}`}
      >
        <div ref={sliderRef} className="keen-slider flex gap-4">
          {items &&
            items.map((item, index) => {
              return (
                <div key={index}>
                  <VerticalNewsCard
                    className="slide"
                    key={(item as INews).id}
                    image={item.image}
                    title={item.title}
                    description={item.description}
                    time={item.time}
                    location={item.location}
                    category={item.category}
                    href={item.href}
                    type={item.type}
                    customBackgroundColor={customBackgroundColor}
                    textColor={textColor || item.textColor}
                  />
                </div>
              );
            })}
        </div>
        {loaded && instanceRef.current && (
          <>
            <SliderNavigationButton
              direction="left"
              onClick={(e: any) => {
                e.stopPropagation() || instanceRef?.current?.prev();
              }}
              disabled={currentIndex === 0}
            />

            <SliderNavigationButton
              direction="right"
              onClick={(e: any) =>
                e.stopPropagation() || instanceRef.current?.next()
              }
              disabled={
                currentIndex ===
                instanceRef.current.track.details.slides.length - 1
              }
            />
          </>
        )}
      </div>
    </div>
  );
};

interface SliderNavigationButtonProps {
  disabled: boolean;
  direction: "left" | "right";
  onClick: (e: any) => void;
}

const SliderNavigationButton: React.FC<SliderNavigationButtonProps> = ({
  disabled,
  direction,
  onClick,
}) => {
  const customTranstion: CSSProperties = {
    transition:
      "transform 0.5s ease-in, opacity 0.35s cubic-bezier(0.15, 0, 0.2, 1) 0.1s",
  };
  return (
    <button
      onClick={onClick}
      style={customTranstion}
      className={`w-12 h-12 absolute top-1/2 -translate-y-1/2 flex justify-center items-center bg-grey-900 shadow-sm rounded-3xl ${
        direction === "left" ? "left-[15px]" : "right-[15px]"
      } ${disabled ? "hidden" : "opacity-50 hover:opacity-100"} `}
    >
      {direction === "left" && <ChevronLeft size={32} className="fill-white" />}
      {direction === "right" && (
        <ChevronRight size={32} className="fill-white" />
      )}
    </button>
  );
};

export default Carousel;
