import React, { Children, Fragment } from "react";
import Grid from "@components/Grid";
import { Link } from "@components/Link";
type Image = {
  src: string;
  alt: string;
  href?: string;
};

type BaseProps = {
  title?: string;
  columns: 2 | 3 | 4 | 5 | 6;
  className?: string;
  customGap?: string;
  customResponsiveColumn?: string;
  newTab?: boolean;
};

type EntriesOrChildren =
  | {
      children: React.ReactNode;
      entries?: never;
    }
  | {
      children?: never;
      entries: Image[];
    };

type Props = BaseProps & EntriesOrChildren;

export const LogoGrid: React.FC<Props> = ({
  title,
  columns,
  className = "",
  customGap = "",
  customResponsiveColumn = "",
  children,
  entries,
  newTab = true,
}) => {
  const GAP_MAPPING: Record<string, string> = {
    2: "gap-10",
    3: "gap-10",
    4: "gap-x-10 gap-y-10 sm:gap-x-12 sm:gap-y-12 xl:gap-x-10 xl:gap-y-10",
    5: "gap-x-16 gap-y-16 sm:gap-x-16 sm:gap-y-12 xl:gap-x-1 xl:gap-y-10",
    6: "gap-x-1 gap-y-14 sm:gap-x-16 sm:gap-y-12 xl:gap-x-1 xl:gap-y-14",
  };
  const RESPONSIVE_COLUMN_MAPPING: Record<string, string> = {
    2: "",
    3: "grid-cols-2",
    4: "grid-cols-2 lg:grid-cols-3",
    5: "grid-cols-2 sm:grid-cols-4",
    6: "grid-cols-4",
  };

  return (
    <div className={className}>
      {title && (
        <h5 className="mb-14 font-plex text-base text-center leading-6 font-semibold tracking-wider text-grey-900 uppercase">
          {title}
        </h5>
      )}
      <Grid
        columns={columns}
        customGap={customGap ? customGap : GAP_MAPPING[columns]}
        customResponsiveColumn={
          customResponsiveColumn
            ? customResponsiveColumn
            : RESPONSIVE_COLUMN_MAPPING[columns]
        }
      >
        {children && Children.map(children, (child) => child)}
        {entries &&
          entries.map((entry, i) => (
            <Fragment key={i}>
              {entry.href ? (
                <Link
                  to={entry.href}
                  {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
                  className={`flex justify-center items-center`}
                >
                  <Logo src={entry.src} alt={entry.alt} />
                </Link>
              ) : (
                <div className={`flex justify-center items-center`}>
                  <Logo src={entry.src} alt={entry.alt} />
                </div>
              )}
            </Fragment>
          ))}
      </Grid>
    </div>
  );
};

interface LogoItemProps {
  src: string;
  alt: string;
  href?: string;
}

export const Logo: React.FC<LogoItemProps> = ({ src, alt, href }) => {
  return (
    <Fragment>
      {href ? (
        <Link to={href}>
          <img className="w-full h-full object-contain" src={src} alt={alt} />
        </Link>
      ) : (
        <img className="w-full h-full object-contain" src={src} alt={alt} />
      )}
    </Fragment>
  );
};
