import React, { Fragment } from "react";
import { Link } from "@components/Link";

type PillType =
  | "success"
  | "danger"
  | "warning"
  | "info"
  | "dark"
  | "default"
  | "purple"
  | "pink"
  | "white";

type BaseProps = {
  title: string;
  rounded?: boolean;
  size: "sm" | "lg";
  className?: string;
  color?: string;
};

type TypeOrCustom =
  | {
      type?: never;
      customBgColor: string;
    }
  | {
      type: PillType;
      customBgColor?: never;
    };

type HrefOrOnClick =
  | {
      href?: string;
      onClick?: never;
      newTab?: boolean;
    }
  | {
      href?: never;
      onClick?: () => void;
      newTab?: never;
    };

type onIconClickOrNot =
  | {
      icon?: React.ReactNode;
      onIconClick?: () => void;
      iconRight?: boolean;
    }
  | {
      icon?: never;
      onIconClick?: never;
      iconRight?: never;
    };

type Props = BaseProps & HrefOrOnClick & onIconClickOrNot & TypeOrCustom;

export const Pill: React.FC<Props> = ({
  type = "default",
  href,
  title,
  icon,
  iconRight,
  rounded,
  size = "lg",
  newTab,
  onClick,
  onIconClick,
  className = "",
  customBgColor = "",
}) => {
  const STYLE_MAPPING: Record<string, string> = {
    success: "bg-teal-100 text-teal-700",
    danger: "bg-red-100 text-red-700",
    warning: "bg-amber-100 text-amber-700",
    info: "bg-blue-50 text-blue-700",
    dark: "bg-grey-900 text-grey-100",
    default: "bg-grey-200 text-grey-800",
    purple: "bg-purple-100 text-purple-800",
    pink: "bg-pink-100 text-pink-800",
    white: "bg-white text-grey-900",
    custom: "",
  };

  const ICON_STYLE_MAPPING: Record<string, string> = {
    success: "text-teal-400",
    danger: "text-red-400",
    warning: "text-amber-400",
    info: "text-blue-400",
    dark: "text-grey-500",
    default: "text-grey-500",
    purple: "text-purple-400",
    pink: "text-pink-400",
    white: "text-grey-900",
    custom: "text-grey-900"
  };

  const SIZE_MAPPING: Record<string, string> = {
    sm: "w-fit flex items-center px-2.5 sm:px-3 py-0.5 text-xs leading-4 font-medium font-plex no-underline not-prose",
    lg: "w-fit flex items-center px-2.5 sm:px-3.5 md:py-1 text-sm leading-5 font-medium font-plex no-underline not-prose",
  };

  const OTHER_STYLES_MAPPING: Record<string, string> = {
    rounded: "rounded-full",
    square: "rounded",
  };

  const iconPositionClasses =
    icon && iconRight
      ? "pr-1.5 sm:pr-1"
      : icon && !iconRight
      ? "pl-1 sm:pl-1.5"
      : "";

  const renderPillContent = () => {
    return (
      <Fragment>
        {icon && (
          <span
            onClick={() => onIconClick && onIconClick()}
            className={`${ICON_STYLE_MAPPING[type]} ${
              iconRight ? "order-2 ml-0.5" : "order-1 mr-0.5"
            } ${onIconClick ? "cursor-pointer" : ""}`}
          >
            {icon}
          </span>
        )}
        <span className={iconRight ? "order-1" : "order-2"}>{title}</span>
      </Fragment>
    );
  };

  return href ? (
    <Link
      to={href}
      style={customBgColor ? { backgroundColor: customBgColor } : {}}
      className={`${STYLE_MAPPING[type]} ${SIZE_MAPPING[size]} ${
        rounded
          ? OTHER_STYLES_MAPPING["rounded"]
          : OTHER_STYLES_MAPPING["square"]
      } ${iconPositionClasses}`}
      {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
    >
      {renderPillContent()}
    </Link>
  ) : (
    <div
      style={customBgColor ? { backgroundColor: customBgColor } : {}}
      onClick={() => onClick && !onIconClick && onClick()}
      className={`${STYLE_MAPPING[type]} ${SIZE_MAPPING[size]} ${
        rounded
          ? OTHER_STYLES_MAPPING["rounded"]
          : OTHER_STYLES_MAPPING["square"]
      } ${iconPositionClasses} ${
        onClick && !onIconClick ? "cursor-pointer" : "cursor-default"
      } ${className}`}
    >
      {renderPillContent()}
    </div>
  );
};
