import React, { CSSProperties } from "react";
import { INewsCard } from "@interfaces/INews";
import { Link } from "@components/Link";
import { Pill } from "@components/Pill";
import { Time, Location, Launch } from "@carbon/icons-react";
type PillColor = "dark" | "white";

export const VerticalNewsCard: React.FC<INewsCard> = ({
  image,
  title,
  description,
  time,
  location,
  category,
  href,
  type = "dark",
  className = "",
  customBackgroundColor,
  textColor = "",
  externalTitle = "",
  newTab,
}) => {
  const inlineStyle: CSSProperties = {
    backgroundColor: customBackgroundColor,
  };
  const PILL_COLOR_MAPPING: Record<string, PillColor> = {
    dark: "white",
    light: "dark",
  };
  const STYLE_MAPPING: Record<string, Record<string, string>> = {
    dark: {
      outerContainer:
        "snap-start w-[280px] sm:w-[380px] md:w-[420px] xl:w-[460px] bg-grey-900",
      contentContainer:
        "group aspect-[56/51] sm:aspect-[100/61] relative p-6 sm:p-7 md:p-8",
    },
    light: {
      outerContainer:
        "snap-start w-[280px] sm:w-[380px] md:w-[420px] xl:w-[460px] bg-pearl-500",
      contentContainer:
        "group aspect-[56/51] sm:aspect-[100/61] relative p-6 sm:p-7 md:p-8",
    },
    custom: {
      outerContainer:
        "snap-start w-[280px] sm:w-[380px] md:w-[420px] xl:w-[460px]",
      contentContainer:
        "group aspect-[56/51] sm:aspect-[100/61] relative p-6 sm:p-7 md:p-8",
    },
  };
  const TEXT_STYLE_MAPPING: Record<string, Record<string, string>> = {
    dark: {
      heading:
        "group-peer-hover:multi-line-underline group-hover:multi-line-underline xl:multi-line-3-white font-karla text-2xl xl:text-3xl leading-8 md:leading-9 font-bold text-white line-clamp-3 sm:line-clamp-2 xl:line-clamp-none",
      paragraph:
        "mt-2 md:mt-3 font-plex text-base md:text-lg leading-6 md:leading-7 font-normal text-grey-100 line-clamp-2 md:line-clamp-2 xl:line-clamp-3",
      event:
        "mt-2 md:mt-3 font-plex text-base md:text-lg leading-6 md:leading-7 font-normal text-grey-50",
    },
    light: {
      heading:
        "group-peer-hover:multi-line-underline group-hover:multi-line-underline xl:multi-line-3-grey-900 font-karla text-2xl xl:text-3xl leading-8 md:leading-9 font-bold text-grey-900 line-clamp-3 sm:line-clamp-2 xl:line-clamp-none",
      paragraph:
        "mt-2 md:mt-4 font-plex text-base md:text-lg leading-6 md:leading-7 font-normal text-grey-900 line-clamp-2 md:line-clamp-2 xl:line-clamp-3",
      event:
        "mt-2 md:mt-4 font-plex text-base md:text-lg leading-6 md:leading-7 font-normal text-grey-900",
    },
  };
  const renderCardContent = (type: "light" | "dark") => {
    if (type === "light") {
      return (
        <p
          className={
            customBackgroundColor
              ? TEXT_STYLE_MAPPING[textColor]["paragraph"]
              : TEXT_STYLE_MAPPING[type]["paragraph"]
          }
        >
          {description}
        </p>
      );
    }
    if (type === "dark") {
      return (
        <div
          className={
            customBackgroundColor
              ? TEXT_STYLE_MAPPING[textColor]["event"]
              : TEXT_STYLE_MAPPING[type]["event"]
          }
        >
          {time && (
            <span className="flex items-center">
              <Time size={16} className="mr-1 md:mr-2" />
              {time}
            </span>
          )}
          {location && (
            <span className="flex items-center mt-1">
              <Location size={16} className="mr-1 md:mr-2" />
              {location}
            </span>
          )}
        </div>
      );
    }
    return null;
  };
  return (
    <div
      style={inlineStyle}
      className={`${
        customBackgroundColor
          ? STYLE_MAPPING["custom"]["outerContainer"]
          : STYLE_MAPPING[type]["outerContainer"]
      } ${className}`}
    >
      {externalTitle ? (
        <div className="peer aspect-square relative">
          <div className="absolute left-1/2 top-1/2 -translate-x-1/2 -translate-y-1/2">
            <Pill
              newTab
              type={PILL_COLOR_MAPPING[type]}
              size="lg"
              rounded
              href={href}
              title={externalTitle}
              icon={<Launch className="w-4 h-4 mr-2" />}
            />
          </div>
          <Link to={href} {...(newTab ? { target: "_blank", rel: "noopener" } : {})}>
            <img
              className="w-full h-full object-cover"
              alt={image.alt}
              src={image.src}
            />
          </Link>
        </div>
      ) : (
        <div className="peer aspect-square">
          <Link to={href} {...(newTab ? { target: "_blank", rel: "noopener" } : {})}>
            <img
              className="w-full h-full object-cover"
              alt={image.alt}
              src={image.src}
            />
          </Link>
        </div>
      )}
      <div
        className={
          customBackgroundColor
            ? STYLE_MAPPING["custom"]["contentContainer"]
            : STYLE_MAPPING[type]["contentContainer"]
        }
      >
        <div className="w-full h-full relative">
          <Link to={href} {...(newTab ? { target: "_blank", rel: "noopener" } : {})}>
            <h5
              className={
                customBackgroundColor
                  ? TEXT_STYLE_MAPPING[textColor]["heading"]
                  : TEXT_STYLE_MAPPING[type]["heading"]
              }
            >
              {title}
            </h5>
            {renderCardContent(type)}
          </Link>
          <div className="absolute bottom-0 left-0 flex flex-wrap">
            <Pill
              type={PILL_COLOR_MAPPING[type]}
              size="sm"
              rounded
              href={category.href}
              title={category.title}
            />
          </div>
        </div>
      </div>
    </div>
  );
};
